export declare function isBun(): boolean;
/**
 * Validate if you're running a compatible minimum version of Bun
 */
export declare function validateRuntime(): void;
/**
 * Returns true if the gravity binary requires an upgrade
 *
 * @param version current version
 * @returns
 */
export declare function validateGravityRequiresUpgrade(version: string): boolean;
//# sourceMappingURL=runtime.d.ts.map