/**
 * Workbench file generator for Vite builds
 */
import type { Logger, WorkbenchConfig } from '../../../types';
/**
 * Generate workbench files in .agentuity/workbench-src/
 * This keeps them separate from user's source code
 */
export declare function generateWorkbenchFiles(rootDir: string, _projectId: string, config: WorkbenchConfig, logger: Logger): Promise<void>;
//# sourceMappingURL=workbench-generator.d.ts.map