export declare const apiSubcommand: import("../../..").SubcommandDefinition;
/**
 * Get the raw prompt content without hash.
 */
export declare function getPromptContent(): string;
/**
 * Generate the API prompt with hash comment.
 */
export declare function generateLLMPrompt(): string;
//# sourceMappingURL=api.d.ts.map