interface GravityClient {
    filename: string;
    version: string;
}
/**
 * Remove previously downloaded gravity version directories after a newer
 * version has started successfully.
 *
 * Safety guard: only removes sibling directories that contain a gravity
 * binary, leaving any unrelated files/folders untouched.
 */
export declare function sweepOldGravityVersions(gravityDir: string, currentVersion: string): string[];
/**
 *
 * @returns full path to the downloaded file
 */
export declare function download(gravityDir: string): Promise<GravityClient>;
export {};
//# sourceMappingURL=download.d.ts.map