import { type PatchModule, buildPatchFilter } from './_util';
export { buildPatchFilter };
export declare function generatePatches(): Map<string, PatchModule>;
export declare function applyPatch(filename: string, patch: PatchModule): Promise<[string, Bun.Loader]>;
//# sourceMappingURL=index.d.ts.map