/**
 * Vite plugin to shim process.env for browser code only
 *
 * Replaces process.env with import.meta.env ONLY in files under src/web
 * to avoid breaking server-side code that needs real process.env
 */
import type { Plugin } from 'vite';
export declare function browserEnvPlugin(): Plugin;
//# sourceMappingURL=browser-env-plugin.d.ts.map