export { getResourceInfo, getResourceRegion, setResourceInfo, setResourceRegion, deleteResourceRegion, clearProfileCache, closeDatabase, type ResourceType, type ResourceInfo, } from './resource-region';
export { getCachedProject, setCachedProject, clearProjectCache } from './project-cache';
export { hasAgentSeenIntro, markAgentIntroSeen, hasAgentSeenInputHint, markAgentInputHintSeen, } from './agent-intro';
export { getCachedUserInfo, setCachedUserInfo, clearCachedUserInfo } from './user-cache';
//# sourceMappingURL=index.d.ts.map